# -*- coding: utf-8 -*-
from PySide import QtCore, QtGui
from PySide.QtDeclarative import *
from PySide.QtGui import QDesktopServices as QDS

import sys
import os
import string
import ConfigParser
import platform

from pwmlib import *



# Description: A Harmattan QML UI for passwordmaker (http://www.passwordmaker.org)
# Copyright:   2011 by George Ruinelli




APP_NAME = 'PasswordMaker'
CONFIG_PATH= os.path.join(QDS.storageLocation(QDS.DataLocation), APP_NAME)
CONFIG_FILE=CONFIG_PATH+"/config.conf"



####################################################################################################
class Configuration():
  def Load(self):
    global CONFIG_FILE
    print "Loading configuration from:", CONFIG_FILE
    self.ConfigParser = ConfigParser.SafeConfigParser()
    try:
      self.ConfigParser.read(CONFIG_FILE)
    except: #use default config
      print "Configuration file "+ CONFIG_FILE + " not existing or not compatible"

    try:
      self.ConfigParser.add_section('main')
    except:
      pass

    try:
      self.username = self.ConfigParser.get("main", "username")
      self.hash = self.ConfigParser.get("main", "hash")
      self.characters = self.ConfigParser.get("main", "characters")
      self.length = int(self.ConfigParser.get("main", "length"))
      self.modifier = self.ConfigParser.get("main", "modifier")
      self.prefix = self.ConfigParser.get("main", "prefix")
      self.suffix = self.ConfigParser.get("main", "suffix")    
      print "Configuration loaded"  
    except:
      print "Error loading configuration"


    
    
    
    
    
    
  def Write(self):
    global CONFIG_PATH, CONFIG_FILE
    print "Write configuration to:", CONFIG_FILE  
    
    #self.ConfigParser.add_section('main')
               
    self.ConfigParser.set('main', 'username', str(self.username))
    self.ConfigParser.set('main', 'hash', str(self.hash))
    self.ConfigParser.set('main', 'characters', str(self.characters))
    self.ConfigParser.set('main', 'length', str(self.length))
    self.ConfigParser.set('main', 'modifier', str(self.modifier))
    self.ConfigParser.set('main', 'prefix', str(self.prefix))
    self.ConfigParser.set('main', 'suffix', str(self.suffix))

    try:
      os.makedirs(CONFIG_PATH)
    except:
      pass

    try:
      handle = open(CONFIG_FILE, 'w')
      self.ConfigParser.write(handle)
      handle.close()
      print "Configuration saved"
    except:
      print "Failed to write configuration file!"
  
  
  
  ConfigParser=None
  username=""
  hash="MD5"
  characters="A-Z, a-z, 0-9, $"
  length=8
  modifier=""
  prefix=""
  suffix=""
  
  
  
   
  
  
  
  
  
####################################################################################################
class Main_frm(QtCore.QObject):
  def __init__(self):
    QtCore.QObject.__init__(self)
    
        
    root = view.rootObject()
    
    #QML objects
    self.txtmasterpassword = root.findChild(QtCore.QObject, 'txtmasterpassword')
    self.txttext = root.findChild(QtCore.QObject, 'txttext')
    
    self.txtusername = root.findChild(QtCore.QObject, 'txtusername')
    self.bhash = root.findChild(QtCore.QObject, 'bhash')
    #self.cbhmac = root.findChild(QtCore.QObject, 'cbhmac')
    self.bcharacters = root.findChild(QtCore.QObject, 'bcharacters')
    self.sllength = root.findChild(QtCore.QObject, 'sllength')
    #self.cbtrim = root.findChild(QtCore.QObject, 'cbtrim')    
    
    self.txtmodifier = root.findChild(QtCore.QObject, 'txtmodifier')
    self.txtprefix = root.findChild(QtCore.QObject, 'txtprefix')
    self.txtsuffix = root.findChild(QtCore.QObject, 'txtsuffix')
    #self.bl33t = root.findChild(QtCore.QObject, 'bl33t')
    #self.sll33tlevel = root.findChild(QtCore.QObject, 'sll33tlevel')   

    self.lblpassword = root.findChild(QtCore.QObject, 'lblpassword')
    self.txtpassword = root.findChild(QtCore.QObject, 'txtpassword')
    
    
    
    
    
  def UpdateScreen(self):
    global config
    #print "config.username:", config.username
    self.txtusername.setProperty('text', config.username)
    self.bhash.setProperty('text', config.hash)
    self.sllength.setProperty('value', config.length)
    self.txtmodifier.setProperty('text', config.modifier)
    self.txtprefix.setProperty('text', config.prefix)
    self.txtsuffix.setProperty('text', config.suffix)
     
    characters=string.replace(config.characters, "$", "`~!@#$%^&*()_-+={}|[]\\:\";'<>?,./")
    self.bcharacters.setProperty('text', characters)
             
    

  @QtCore.Slot() #needed, so a QML opject can send a signal
  def values_changed(self):
    #print "text field values changed"    
    self.generate()
    
        
        
  @QtCore.Slot(result=str) #needed, so a QML opject can send a signal
  def get_version(self):
    global version, build
    return str(version) + "-" + str(build)
    
    
    
  #@QtCore.Slot() #needed, so a QML opject can send a signal
  #def quit(self):
    #quit()


  def generate(self):
    print "generate..."
    
    #Get data from text fields
    masterpassword=self.txtmasterpassword.property('text')
    text=self.txttext.property('text')
        
    account=self.txtusername.property('text')
    hash=self.bhash.property('text')
    #hmac=self.cbhmac.property('checked')
    
    characters=self.bcharacters.property('text')
    length=self.sllength.property('value')
    #trim=self.cbtrim.property('checked')
    
    modifier=self.txtmodifier.property('text')
    prefix=self.txtprefix.property('text')
    suffix=self.txtsuffix.property('text')
    #l33t=self.bl33t.property('text')
    #l33tlevel=self.sll33tlevel.property('value')

    #print masterpassword, text, account, hash, characters, length, trim, modifier, prefix, suffix, l33t, l33t_level
    #print masterpassword, text, account, hash, characters, length, modifier, prefix, suffix
    
      
    config.username=account
    config.hash=hash
    config.characters=string.replace(characters, "`~!@#$%^&*()_-+={}|[]\\:\";'<>?,./", "$")
    config.length=int(length)
    config.modifier=modifier
    config.prefix=prefix
    config.suffix=suffix
    
    #config.Write()
    
    
    
    
    

    if(masterpassword == ""): return #do not generate password if no masterpassword is entered


    length=int(length)
    #l33tlevel=int(l33tlevel)
    
    #print characters    
    characters=string.replace(characters, "A-Z, a-z, 0-9, `~!@#$%^&*()_-+={}|[]\\:\";'<>?,./", "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789`~!@#$%^&*()_-+={}|[]\\:\";'<>?,./")
    characters=string.replace(characters, "0-9, a-f", "0123456789abcdef")
    characters=string.replace(characters, "0-9", "0123456789")
    #characters=string.replace(characters, "`~!@#$%^&*()_-+={}|[]\\:\";'<>?,./", "`~!@#$%^&*()_-+={}|[]\\:\";'<>?,./") //not needed, no string replacements
    
    #print characters
    
    hash=string.replace(hash, "MD4", "md4")
    hash=string.replace(hash, "MD5", "md5")
    hash=string.replace(hash, "SHA1", "sha1")
    hash=string.replace(hash, "SHA256", "sha256")
    hash=string.replace(hash, "RIPEMD160", "rmd160")
    
    
    
    

    #if(str(hmac)=="True"): hash = "hmac-"+hash    
    ##Note: The following parameters seem not to be supported by the python pwmlib!
    #	account
    #	trim
    #	hmac
    #	l33t, l33tlevel
	
    l33t="False"
    l33tlevel=-1
	
    text=text+account+modifier 
	
	
	
	
    
        
    ######## FROM THE DOCUMENTATION OF PASSWORD MAKER: ###############
    #If a command-line argument is missing or not specified in the "Defaults" of
    #passwordmaker.rdf or the file pointed to by -f/--file, these built-in defaults are
    #used in its place:

	#masterPassword : none
	#url            : none
	#algorithm      : MD5
	#hmac           : no
	#trim           : yes
	#length         : 8
	#characters     : ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789`~!@#$%^&*()_-+={}|[]\:";'<>?,./
	#useLeet        : none
	#leetLevel      : 0
	#username       : none
	#modifier       : none
	#prefix         : none
	#suffix         : none



    try:
      PWmaker = PWM()
      #(self,hashAlgorithm, key, data, whereToUseL33t, l33tLevel, passwordLength, charset, prefix="", suffix=""):
      p=PWmaker.generatepassword(	
		hash,
		masterpassword,
		text,
		l33t,
		l33tlevel,
		length,
		characters,
		prefix,
		suffix)

      self.lblpassword.setProperty('text', p) #Set data on label
      self.txtpassword.setProperty('text', p) #Set data on label, so we can copy it easely to system clipboard
    except PWM_Error, e:
        print e
        self.lblpassword.setProperty('text', "<i>!!ERROR!!</i>") #Set data on label
        self.txtpassword.setProperty('text', "")
        #sys.exit(1)










####################################################################################################

print "Loading", APP_NAME, "..."

#Load version file
try:
    file = open("version", 'r')
    version = file.readline()
    version=version[:-1]
    build = file.readline()
    print "Version: "+str(version)+ "-"+str(build)
except:
    print "Version file not found, please check your installation!"



app = QtGui.QApplication(sys.argv)

view = QDeclarativeView()
view.setResizeMode(QDeclarativeView.SizeRootObjectToView)
view.setSource("qml/main.qml")

if(platform.machine().startswith('arm')): view.showFullScreen()
view.show()

mainform = Main_frm()

config = Configuration()
config.Load()

mainform.UpdateScreen()

view.rootContext().setContextProperty('mainform', mainform)

app.exec_()

print "app closing.."
config.Write()